// *****************************************************************************
// Notices:
//
// Copyright © 2019, 2021 United States Government as represented by the Administrator
// of the National Aeronautics and Space Administration. All Rights Reserved.
//
// Disclaimers
//
// No Warranty: THE SUBJECT SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY OF
// ANY KIND, EITHER EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT LIMITED
// TO, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL CONFORM TO SPECIFICATIONS,
// ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
// OR FREEDOM FROM INFRINGEMENT, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL BE
// ERROR FREE, OR ANY WARRANTY THAT DOCUMENTATION, IF PROVIDED, WILL CONFORM TO
// THE SUBJECT SOFTWARE. THIS AGREEMENT DOES NOT, IN ANY MANNER, CONSTITUTE AN
// ENDORSEMENT BY GOVERNMENT AGENCY OR ANY PRIOR RECIPIENT OF ANY RESULTS,
// RESULTING DESIGNS, HARDWARE, SOFTWARE PRODUCTS OR ANY OTHER APPLICATIONS
// RESULTING FROM USE OF THE SUBJECT SOFTWARE.  FURTHER, GOVERNMENT AGENCY
// DISCLAIMS ALL WARRANTIES AND LIABILITIES REGARDING THIRD-PARTY SOFTWARE, IF
// PRESENT IN THE ORIGINAL SOFTWARE, AND DISTRIBUTES IT ''AS IS.''
//
// Waiver and Indemnity:  RECIPIENT AGREES TO WAIVE ANY AND ALL CLAIMS AGAINST
// THE UNITED STATES GOVERNMENT, ITS CONTRACTORS AND SUBCONTRACTORS, AS WELL AS
// ANY PRIOR RECIPIENT.  IF RECIPIENT'S USE OF THE SUBJECT SOFTWARE RESULTS IN
// ANY LIABILITIES, DEMANDS, DAMAGES, EXPENSES OR LOSSES ARISING FROM SUCH USE,
// INCLUDING ANY DAMAGES FROM PRODUCTS BASED ON, OR RESULTING FROM, RECIPIENT'S
// USE OF THE SUBJECT SOFTWARE, RECIPIENT SHALL INDEMNIFY AND HOLD HARMLESS THE
// UNITED STATES GOVERNMENT, ITS CONTRACTORS AND SUBCONTRACTORS, AS WELL AS ANY
// PRIOR RECIPIENT, TO THE EXTENT PERMITTED BY LAW.  RECIPIENT'S SOLE REMEDY FOR
// ANY SUCH MATTER SHALL BE THE IMMEDIATE, UNILATERAL TERMINATION OF THIS
// AGREEMENT.
// *****************************************************************************
import React from 'react';
import classNames from 'classnames';
import PropTypes from 'prop-types';
import { withStyles } from '@material-ui/core/styles';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import TablePagination from '@material-ui/core/TablePagination';
import TableRow from '@material-ui/core/TableRow';
import TableSortLabel from '@material-ui/core/TableSortLabel';
import Toolbar from '@material-ui/core/Toolbar';
import Typography from '@material-ui/core/Typography';
import Paper from '@material-ui/core/Paper';
import Checkbox from '@material-ui/core/Checkbox';
import IconButton from '@material-ui/core/IconButton';
import Tooltip from '@material-ui/core/Tooltip';
import Snackbar from '@material-ui/core/Snackbar';
import Button from '@material-ui/core/Button';
import ImageList from '@material-ui/core/ImageList';
import ImageListItem from '@material-ui/core/ImageListItem';

import CloseIcon from '@material-ui/icons/Close';
import DeleteIcon from '@material-ui/icons/Delete';
import ListIcon from '@material-ui/icons/List';
// import EditIcon from '@material-ui/icons/Edit';
import AddIcon from '@material-ui/icons/AddCircle';
// status icons
import InProgressIcon from '@material-ui/icons/MoreHoriz';
import RemoveIcon from '@material-ui/icons/Remove';
import PauseIcon from '@material-ui/icons/Pause';
import CompletedIcon from '@material-ui/icons/Done';
import AttentionIcon from '@material-ui/icons/PriorityHigh';
import { lighten } from '@material-ui/core/styles/colorManipulator';

import DisplayRequirementDialog from './DisplayRequirementDialog';
import CreateRequirementDialog from './CreateRequirementDialog';
import DeleteRequirementDialog from './DeleteRequirementDialog';

// select and menu for status column
import Select from '@material-ui/core/Select';
import MenuItem from '@material-ui/core/MenuItem';

import ExportIcon from '@material-ui/icons/ArrowUpward';

import VariablesView from './VariablesView';
import * as d3 from "d3";
import {getRequirementStyle} from "../utils/utilityFunctions";

const constants = require('../parser/Constants');
const sharedObj = require('electron').remote.getGlobal('sharedObj');

const db = sharedObj.db;
const app = require('electron').remote.app;
const system_dbkeys = sharedObj.system_dbkeys;


let counter = 0;
// status is also saved in database
function createData(dbkey, rev, reqid, summary, project, status, semantics, fulltext) {
  counter += 1;
  return { rowid: counter, dbkey, rev, reqid, summary, project, status: status || 'None', semantics, fulltext};
}

function desc(a, b, orderBy) {
  var element_a, element_b
  if (rows.find(r => r.id == orderBy).numeric) {
    element_a = a[orderBy]
    element_b = b[orderBy]
  } else {
    element_a = a[orderBy] ? a[orderBy].toLowerCase().trim(): 'NONE';
    element_b = b[orderBy] ? b[orderBy].toLowerCase().trim(): 'NONE';
  }

  if (element_b < element_a)
    return -1
  if (element_b > element_a)
    return 1
  return 0
}

function stableSort(array, cmp) {
  const stabilizedThis = array.map((el, index) => [el, index]);
  stabilizedThis.sort((a, b) => {
    const order = cmp(a[0], b[0]);
    if (order !== 0) return order;
    return a[1] - b[1];
  });
  return stabilizedThis.map(el => el[0]);
}

function getSorting(order, orderBy) {
  return order === 'desc' ? (a, b) => desc(a, b, orderBy) : (a, b) => -desc(a, b, orderBy);
}

const rows = [
  { id: 'status', numeric: false, disablePadding: false, label: 'Status' },
  { id: 'reqid', numeric: false, disablePadding: false, label: 'ID' },
  { id: 'add', numeric: false, disablePadding: false, label: '' },
  { id: 'summary', numeric: false, disablePadding: false, label: 'Summary' },
  { id: 'project', numeric: false, disablePadding: false, label: 'Project' },
]

class SortableTableHead extends React.Component {
  createSortHandler = property => event => {
    this.props.onRequestSort(event, property);
  };

  render() {
    const { onSelectAllClick, order, orderBy, numSelected, rowCount, enableBulkChange } = this.props;

    return (
      <TableHead>
        <TableRow>
          {
            enableBulkChange &&
            <TableCell padding="checkbox">
              <Checkbox
                indeterminate={numSelected > 0 && numSelected < rowCount}
                checked={numSelected === rowCount}
                onChange={onSelectAllClick}
              />
            </TableCell>
          }
          {rows.map(row => {
            return (
              <TableCell
                key={row.id}
                align={row.numeric?'right':'left'}
                sortDirection={orderBy === row.id ? order : false}
              >
                <Tooltip
                  title="Sort"
                  placement={row.numeric ? 'bottom-end' : 'bottom-start'}
                  enterDelay={300}
                >
                  <TableSortLabel
                    active={orderBy === row.id}
                    direction={order}
                    onClick={this.createSortHandler(row.id)}
                  >
                    {row.label}
                  </TableSortLabel>
                </Tooltip>
              </TableCell>
            );
          }, this)}
        </TableRow>
      </TableHead>
    );
  }
}

SortableTableHead.propTypes = {
  numSelected: PropTypes.number.isRequired,
  onRequestSort: PropTypes.func.isRequired,
  onSelectAllClick: PropTypes.func.isRequired,
  order: PropTypes.string.isRequired,
  orderBy: PropTypes.string.isRequired,
  rowCount: PropTypes.number.isRequired,
  enableBulkChange: PropTypes.bool.isRequired
};

const toolbarStyles = theme => ({
  root: {
    paddingRight: theme.spacing(),
  },
  highlight:
    theme.palette.type === 'light'
      ? {
          color: theme.palette.secondary.main,
          backgroundColor: lighten(theme.palette.secondary.light, 0.85),
        }
      : {
          color: theme.palette.text.primary,
          backgroundColor: theme.palette.secondary.dark,
        },
  spacer: {
    flex: '1 1 100%',
  },
  actions: {
    color: theme.palette.text.secondary,
  },
  title: {
    flex: '0 0 auto',
  },
  toolbar: {
    display: 'flex',
    flexWrap:'nowrap'
  }
});

let TableToolbar = props => {
  const { numSelected, classes, enableBulkChange, bulkChangeEnabler, deleteSelection } = props;

  return (
    <Toolbar
      className={classNames(classes.root, {
        [classes.highlight]: numSelected > 0,
      })}
    >
      <div className={classes.title}>
        {
          enableBulkChange &&
          <Typography color="inherit" variant="subtitle1">
            {numSelected} selected
          </Typography>
        }
      </div>
      <div className={classes.spacer} />
      <div className={classes.actions}>
        {numSelected > 0 ? (
          <div className={classes.toolbar}>
            <Tooltip title="Delete">
              <IconButton
                aria-label="Delete"
                onClick={() =>  deleteSelection()}>
                <DeleteIcon />
              </IconButton>
            </Tooltip>
            <Tooltip title="Exit Bulk Change">
              <IconButton aria-label="Close Bulk Change" onClick={() => bulkChangeEnabler()}>
                <CloseIcon />
              </IconButton>
            </Tooltip>
          </div>
        ) : (
          <div className={classes.toolbar}>
          <IconButton aria-label="Bulk Change" onClick={() => bulkChangeEnabler()}>
            <Tooltip title="Bulk Change">
            <ListIcon color='secondary'/>
            </Tooltip>
          </IconButton>
          </div>
        )}
      </div>
    </Toolbar>
  );
};

TableToolbar.propTypes = {
  classes: PropTypes.object.isRequired,
  numSelected: PropTypes.number.isRequired,
  enableBulkChange: PropTypes.bool.isRequired,
  bulkChangeEnabler: PropTypes.func.isRequired,
};

TableToolbar = withStyles(toolbarStyles)(TableToolbar);

const styles = theme => ({
  root: {
    width: '100%',
    marginTop: theme.spacing(3),
  },
  table: {
    minWidth: 1020,
  },
  tableWrapper: {
    overflowX: 'auto',
  },
  // style for status select button
  select: {
    borderStyle: 'None',
    borderWidth: 1,
    borderRadius: 5,
    width: 45,
    height: 30,
  }
});

// color to match select background to requirement bubble color
const COLOR_RANGE = ["hsl(0, 0%, 80%)", "hsl(0, 0%, 20%)"]
const colorRange= d3.scaleLinear()
  .domain([1, 7])
  .range(COLOR_RANGE)
  .interpolate(d3.interpolateHcl);

class SortableTable extends React.Component {
  state = {
    order: 'asc',
    orderBy: 'reqid',
    selected: [],
    data: [],
    page: 0,
    rowsPerPage: 10,
    displayRequirementOpen: false,
    selectedRequirement: {},
    selectionBulkChange: [],
    snackBarDisplayInfo: {},
    addChildRequirementMode: undefined,
    createDialogOpen: false,
    deleteDialogOpen: false,
    snackbarOpen: false,
    selectedProject: 'All Projects',
    bulkChangeMode: false,
    deleteUsingCheckBoxes: false,
  };

  constructor(props){
    super(props);
  }

  componentDidMount() {
    this.mounted = true;
    this.formatData();
  }

  componentWillUnmount() {
    this.mounted = false;
  }

  componentDidUpdate(prevProps) {
    if (this.props.selectedProject !== prevProps.selectedProject) {
      this.formatData()
      this.setState(
        {
          selected: [],
          bulkChangeMode: false
        });
    }
    if(this.props.requirements !== prevProps.requirements) {
      this.formatData()
    }
  }

  formatData() {
    if (!this.mounted) return;

    const { selectedProject, requirements } = this.props;
    const filterOff = selectedProject == 'All Projects'
    const data = requirements.filter(r => filterOff || r.doc.project == selectedProject)
        .map(r => {
          return createData(r.doc._id, r.doc._rev, r.doc.reqid, r.doc.fulltext, r.doc.project, r.doc.status, r.doc.semantics, r.doc.fulltext);
        });
      this.setState({
        data,
      })
  }

  handleEnableBulkChange = () => {
    // Clear selection when exiting bulk change mode
    if (this.state.bulkChangeMode) {
      this.state.selected = []
    }
    this.setState({
      bulkChangeMode : !this.state.bulkChangeMode
    })
  }

  handleDeleteSelectedRequirements = () => {
    const { selected, data } = this.state
    this.setState({
      deleteDialogOpen: true,
      deleteUsingCheckBoxes: true,
      selectionBulkChange: data.filter(r => selected.includes(r.dbkey))
    })
  }

  handleRequirementDialogOpen = (row) => event => {event.stopPropagation();
    if (row.dbkey) {
      db.get(row.dbkey).then((doc) => {
        doc.dbkey = row.dbkey
        doc.rev = row.rev
        this.setState({
          selectedRequirement: doc,
          displayRequirementOpen: true,
        })
      }).catch((err) => {
        console.log(err);
      });
    }
  }

  handleRequirementDialogClose = () => {
    this.setState({
      displayRequirementOpen: false,
      addChildRequirement: false
    })
  }

  handleCreateDialogOpen = () => {
    this.setState({
      createDialogOpen: true
    })
  }

  handleAddChildRequirement = (selectedReqId, parentProject) => event => {event.stopPropagation();
    this.setState({
      createDialogOpen: true,
      selectedRequirement: {},
      addChildRequirementMode: {
        parentReqId: selectedReqId,
        parentProject: parentProject
      }
    })
  }

  handleDeleteDialogClose = () => {
    this.setState({
      deleteDialogOpen: false,
      deleteUsingCheckBoxes: false
    })
    // Clear selection when exiting bulk change mode
    if (this.state.bulkChangeMode) {
      this.setState({
        selected : []
      })
    }
  }

  handleDeleteDialogOpen = () => {
    this.setState({
      deleteDialogOpen: true
    })
  }

  handleCreateDialogClose = (requirementUpdated, newReqId, actionLabel) => {
    this.setState({
      createDialogOpen: false,
      snackbarOpen: requirementUpdated,
      snackBarDisplayInfo: {
        modifiedReqId: newReqId,
        action: actionLabel
      },
      addChildRequirementMode: undefined
    })
  }

  handleSnackbarClose = (event, reason) => {
    if (reason === 'clickaway') {
      return;
    }

    this.setState({ snackbarOpen: false });
  };

  handleRequestSort = (event, property) => {
    const orderBy = property;
    let order = 'desc';

    if (this.state.orderBy === property && this.state.order === 'desc') {
      order = 'asc';
    }

    this.setState({ order, orderBy });
  };

  handleSelectAllClick = event => {
    if (event.target.checked) {
      this.setState(state => ({ selected: state.data.map(n => n.dbkey) }));
      return;
    }
    this.setState({ selected: [] });
  };

  handleClick = (event, id) => {
    const { selected } = this.state;
    const selectedIndex = selected.indexOf(id);
    let newSelected = [];

    if (selectedIndex === -1) {
      newSelected = newSelected.concat(selected, id);
    } else if (selectedIndex === 0) {
      newSelected = newSelected.concat(selected.slice(1));
    } else if (selectedIndex === selected.length - 1) {
      newSelected = newSelected.concat(selected.slice(0, -1));
    } else if (selectedIndex > 0) {
      newSelected = newSelected.concat(
        selected.slice(0, selectedIndex),
        selected.slice(selectedIndex + 1),
      );
    }

    this.setState({ selected: newSelected });
  };

  handleChangePage = (event, page) => {
    this.setState({ page });
  };

  handleChangeRowsPerPage = event => {
    this.setState({ rowsPerPage: event.target.value });
  };

  isSelected = id => this.state.selected.indexOf(id) !== -1;

  // user select a status option from menu item
  handleChange = (event, row) => {
    event.stopPropagation();
    if (row.dbkey) {
      db.get(row.dbkey).then(function (doc) {
        return db.put({ ...doc, status: event.target.value }, err => {
          if (err) {
            return console.log(err);
          }
        });
      })
    }
  };

  render() {
    const { classes, selectedProject, existingProjectNames } = this.props;
    const { data, order, orderBy, selected, rowsPerPage, page, bulkChangeMode,
       snackBarDisplayInfo, selectionBulkChange, selectedRequirement, deleteUsingCheckBoxes } = this.state;
    const emptyRows = rowsPerPage - Math.min(rowsPerPage, data.length - page * rowsPerPage);
    const title = 'Requirements: ' + selectedProject
    const selectionForDeletion = deleteUsingCheckBoxes ? selectionBulkChange : [selectedRequirement]

    return (
      <div>
      <Typography variant='h6'>{title}
      </Typography>
      <Paper className={classes.root}>
        <TableToolbar
          numSelected={selected.length}
          enableBulkChange={bulkChangeMode}
          bulkChangeEnabler={this.handleEnableBulkChange}
          deleteSelection={this.handleDeleteSelectedRequirements}/>
        <div className={classes.tableWrapper}>
          <Table className={classes.table} aria-labelledby="tableTitle" size="small">
            <SortableTableHead
              numSelected={selected.length}
              order={order}
              orderBy={orderBy}
              onSelectAllClick={this.handleSelectAllClick}
              onRequestSort={this.handleRequestSort}
              rowCount={data.length}
              enableBulkChange={bulkChangeMode}
            />
            <TableBody>{
                stableSort(data, getSorting(order, orderBy))
                .slice(page * rowsPerPage, page * rowsPerPage + rowsPerPage)
                .map(n => {
                  const isSelected = this.isSelected(n.dbkey);
                  const label = n.reqid ? n.reqid : 'NONE'
                  const projectLabel = n.project ? n.project : 'NONE'
                  // getting requirement bubble color
                  const status = n.status;
                  const colorStyle = getRequirementStyle(n,false);
                  if (this.state.bulkChangeMode) {
                    return (
                      <TableRow
                        hover
                        onClick={event => this.handleClick(event, n.dbkey)}
                        role="checkbox"
                        aria-checked={isSelected}
                        tabIndex={-1}
                        key={n.dbkey}
                        selected={isSelected}
                      >
                        <TableCell padding="checkbox">
                          <Checkbox checked={isSelected} />
                        </TableCell>
                        <TableCell >
                          <Select
                            className={`${classes.select} ${colorStyle}`}
                            disableUnderline
                            value={status}
                            onChange={(event) => this.handleChange(event, n)}
                            onClick={event => event.stopPropagation()}
                          >
                            <MenuItem value="None"/>
                            <MenuItem value={'in progress'}>
                              <Tooltip title="In progress"><InProgressIcon/></Tooltip>
                            </MenuItem>
                            <MenuItem value={'paused'}>
                              <Tooltip title="Paused"><PauseIcon/></Tooltip>
                            </MenuItem>
                            <MenuItem value={'completed'}>
                              <Tooltip title="Completed"><CompletedIcon/></Tooltip>
                            </MenuItem>
                            <MenuItem value={'attention'}>
                              <Tooltip title="Attention"><AttentionIcon/></Tooltip>
                            </MenuItem>
                            <MenuItem value={'deprecated'}>
                              <Tooltip title="Deprecated"><CloseIcon/></Tooltip>
                            </MenuItem>
                          </Select>
                        </TableCell>
                        <TableCell>
                        <Button color='secondary' onClick={this.handleRequirementDialogOpen(n)}>
                            {label}
                          </Button>
                        </TableCell>
                        <TableCell>
                          <Tooltip title="Add Child Requirement">
                            <IconButton
                              aria-label="Add Child Requirement"
                              onClick={this.handleAddChildRequirement(n.reqid, n.project)}>
                              <AddIcon/>
                            </IconButton>
                          </Tooltip>
                        </TableCell>
                        <TableCell>{n.summary}</TableCell>
                        <TableCell>{projectLabel}</TableCell>
                      </TableRow>
                    );
                  } else {
                    return (
                      <TableRow key={n.rowid}>
                        <TableCell >
                          <Select
                            className={`${classes.select} ${colorStyle}`}
                            disableUnderline
                            value={status}
                            onChange={(event) => this.handleChange(event, n)}
                          >
                            <MenuItem value="None"/>
                            <MenuItem value={'in progress'}>
                              <Tooltip title="In progress"><InProgressIcon/></Tooltip>
                            </MenuItem>
                            <MenuItem value={'paused'}>
                              <Tooltip title="Paused"><PauseIcon/></Tooltip>
                            </MenuItem>
                            <MenuItem value={'completed'}>
                              <Tooltip title="Completed"><CompletedIcon/></Tooltip>
                            </MenuItem>
                            <MenuItem value={'attention'}>
                              <Tooltip title="Attention"><AttentionIcon/></Tooltip>
                            </MenuItem>
                            <MenuItem value={'deprecated'}>
                              <Tooltip title="Deprecated"><CloseIcon/></Tooltip>
                            </MenuItem>
                          </Select>
                        </TableCell>
                        <TableCell>
                        <Button color='secondary' onClick={this.handleRequirementDialogOpen(n)}>
                              {label}
                            </Button>
                          </TableCell>
                          <TableCell>
                            <Tooltip title="Add Child Requirement">
                              <IconButton
                                aria-label="Add Child Requirement"
                                onClick={this.handleAddChildRequirement(n.reqid, n.project)}>
                                <AddIcon />
                              </IconButton>
                            </Tooltip>
                          </TableCell>
                        <TableCell>{n.summary}</TableCell>
                        <TableCell>{projectLabel}</TableCell>
                      </TableRow>
                    )
                  }
                })}
              {emptyRows > 0 && (
                <TableRow style={{ height: 49 * emptyRows }}>
                  <TableCell colSpan={6} />
                </TableRow>
              )}
            </TableBody>
          </Table>
        </div>
        <TablePagination
          component="div"
          count={data.length}
          rowsPerPage={rowsPerPage}
          page={page}
          backIconButtonProps={{
            'aria-label': 'Previous Page',
          }}
          nextIconButtonProps={{
            'aria-label': 'Next Page',
          }}
          onPageChange={this.handleChangePage}
          onRowsPerPageChange={this.handleChangeRowsPerPage}
        />
      </Paper>
      <DisplayRequirementDialog
        selectedRequirement={this.state.selectedRequirement}
        open={this.state.displayRequirementOpen}
        handleDialogClose={this.handleRequirementDialogClose}
        handleCreateDialogOpen={this.handleCreateDialogOpen}
        handleDeleteDialogClose={this.handleDeleteDialogClose}
        handleDeleteDialogOpen={this.handleDeleteDialogOpen}/>
      <CreateRequirementDialog
        open={this.state.createDialogOpen}
        handleCreateDialogClose={this.handleCreateDialogClose}
        selectedProject={this.state.selectedProject}
        editRequirement={this.state.selectedRequirement}
        addChildRequirementToParent={this.state.addChildRequirementMode}
        existingProjectNames={this.props.existingProjectNames}
        requirements = {this.props.requirements} />
      <DeleteRequirementDialog
        open={this.state.deleteDialogOpen}
        requirementsToBeDeleted={selectionForDeletion}
        handleDialogClose={this.handleDeleteDialogClose}
      />
      <Snackbar
        anchorOrigin={{
          vertical: 'bottom',
          horizontal: 'center',
        }}
        open={this.state.snackbarOpen}
        autoHideDuration={6000}
        onClose={this.handleSnackbarClose}
        snackbarcontentprops={{
          'aria-describedby': 'message-id',
        }}
        message={<span id="message-id">Requirement Updated</span>}
        action={[
          <Button key="undo" color="secondary" size="small" onClick={this.handleSnackbarClose}>
            {this.state.snackBarDisplayInfo.modifiedReqId}
          </Button>,
          <IconButton
            key="close"
            aria-label="Close"
            color="inherit"
            onClick={this.handleSnackbarClose}
          >
            <CloseIcon />
          </IconButton>,
        ]} />
      </div>
    );
  }
}

SortableTable.propTypes = {
  classes: PropTypes.object.isRequired,
  selectedProject: PropTypes.string.isRequired,
  existingProjectNames: PropTypes.array.isRequired,
  requirements: PropTypes.array.isRequired
};

export default withStyles(styles)(SortableTable);
